@@ -37,15 +37,6 @@
 #include "location.h"
 #include "util.h"
 
-/* Maximum amount to read from an I/O channel at one time */
-
-#define READ_BUF_LEN 4096
-
-#ifndef SUN_LEN
-#define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path)	      \
-		      + strlen ((ptr)->sun_path))
-#endif
-
 static GtkObjectClass *parent_class;
 
 enum {
@@ -64,126 +55,68 @@ struct _ConfigLogEntry
 	gchar      *backend_id;
 };
 
-struct _IOBuffer 
-{
-	GIOChannel  *channel;
-	char         buffer[READ_BUF_LEN + 1];
-	gchar       *read_ptr;
-	gchar       *write_ptr;
-	gboolean     from_socket;
-	gboolean     read_cycle_done;
-	gboolean     closed;
-};
-
-struct _Slave
-{
-	ConfigLog   *config_log;
-	IOBuffer    *buffer;
-	guint        source_id;
-};
-
 struct _ConfigLogPrivate 
 {
 	Location    *location;
 
-	IOBuffer    *file_buffer;
+	FILE        *file_stream;
 	char        *filename;
 	gboolean     deleted;
 
 	GList       *log_data;
 	GList       *first_old;
-
-	char        *socket_filename;
-	IOBuffer    *socket_buffer;
-	gboolean     socket_owner;
-	guint        input_id;
-
-	GList       *slaves;
 };
 
-static void       config_log_init               (ConfigLog *config_log);
-static void       config_log_class_init         (ConfigLogClass *klass);
+static void       config_log_init               (ConfigLog       *config_log);
+static void       config_log_class_init         (ConfigLogClass  *klass);
 
-static void       config_log_set_arg            (GtkObject *object,
-						 GtkArg *arg,
-						 guint arg_id);
+static void       config_log_set_arg            (GtkObject       *object,
+						 GtkArg          *arg,
+						 guint            arg_id);
 
-static void       config_log_get_arg            (GtkObject *object,
-						 GtkArg *arg,
-						 guint arg_id);
+static void       config_log_get_arg            (GtkObject       *object,
+						 GtkArg          *arg,
+						 guint            arg_id);
 
-static void       config_log_destroy            (GtkObject *object);
-static void       config_log_finalize           (GtkObject *object);
+static void       config_log_destroy            (GtkObject       *object);
+static void       config_log_finalize           (GtkObject       *object);
 
-static GList     *find_config_log_entry_id      (ConfigLog *config_log,
-						 GList *start, 
-						 gint id);
-static GList     *find_config_log_entry_date    (ConfigLog *config_log,
-						 GList *start, 
-						 struct tm *date);
-static GList     *find_config_log_entry_backend (ConfigLog *config_log,
-						 GList *start, 
-						 gchar *backend_id);
+static GList     *find_config_log_entry_id      (ConfigLog       *config_log,
+						 GList           *start, 
+						 gint             id);
+static GList     *find_config_log_entry_date    (ConfigLog       *config_log,
+						 GList           *start, 
+						 struct tm       *date);
+static GList     *find_config_log_entry_backend (ConfigLog       *config_log,
+						 GList           *start, 
+						 const gchar     *backend_id);
 
-static GList     *load_log_entry                (ConfigLog *config_log,
-						 gboolean from_socket,
-						 IOBuffer *input,
-						 GList *last);
+static GList     *load_log_entry                (ConfigLog       *config_log,
+						 GList           *last);
 
-static gboolean   parse_line                    (char *buffer, 
-						 int *id, 
-						 struct tm *time, 
-						 char **backend_id);
-static gboolean   time_geq                      (struct tm *time1, 
-						 struct tm *time2);
+static gboolean   parse_line                    (char            *buffer, 
+						 int             *id, 
+						 struct tm       *time, 
+						 char           **backend_id);
+static gboolean   time_geq                      (struct tm       *time1, 
+						 struct tm       *time2);
 
-static gboolean   do_load                       (ConfigLog *config_log);
-static void       do_unload                     (ConfigLog *config_log,
-						 gboolean write_log);
+static gboolean   do_load                       (ConfigLog       *config_log);
+static void       do_unload                     (ConfigLog       *config_log,
+						 gboolean         write_log);
 
-static gint       get_next_id                   (ConfigLog *config_log);
+static gint       get_next_id                   (ConfigLog       *config_log);
 static struct tm *get_beginning_of_time         (void);
 static struct tm *get_current_date              (void);
-static void       write_log                     (IOBuffer *output, 
-						 ConfigLogEntry *entry);
-static void       dump_log                      (ConfigLog *config_log);
-static gboolean   has_nondefaults               (ConfigLog *config_log);
+static void       write_log                     (FILE            *output, 
+						 ConfigLogEntry  *entry);
+static void       dump_log                      (ConfigLog       *config_log);
+static gboolean   has_nondefaults               (ConfigLog       *config_log);
 
-static gboolean   connect_socket                (ConfigLog *config_log);
-static gboolean   check_socket_filename         (ConfigLog *config_log);
-static gboolean   bind_socket                   (ConfigLog *config_log,
-						 int fd,
-						 gboolean do_connect);
-static void       disconnect_socket             (ConfigLog *config_log);
-static gboolean   socket_connect_cb             (GIOChannel *channel,
-						 GIOCondition condition,
-						 ConfigLog *config_log);
-static gboolean   socket_data_cb                (GIOChannel *channel,
-						 GIOCondition condition,
-						 ConfigLog *config_log);
+static void       config_log_entry_destroy      (ConfigLogEntry  *entry);
 
-static void       config_log_entry_destroy      (ConfigLogEntry *entry);
-
-static Slave     *slave_new                     (ConfigLog *config_log,
-						 int fd);
-static void       slave_destroy                 (Slave *slave);
-
-static gboolean   slave_data_cb                 (GIOChannel *channel,
-						 GIOCondition condition,
-						 Slave *slave);
-static void       slave_broadcast_data          (Slave *slave,
-						 ConfigLog *config_log);
-
-static IOBuffer  *io_buffer_new                 (GIOChannel *channel,
-						 gboolean from_socket);
-static void       io_buffer_destroy             (IOBuffer *buffer);
-static int        io_buffer_cycle               (IOBuffer *buffer);
-static gchar     *io_buffer_read_line           (IOBuffer *buffer);
-static void       io_buffer_write               (IOBuffer *buffer,
-						 gchar *str);
-static void       io_buffer_rewind              (IOBuffer *buffer);
-static void       io_buffer_dump                (IOBuffer *source,
-						 IOBuffer *dest);
+static void       dump_file                     (FILE            *input,
+						 FILE            *output);
 
 guint
 config_log_get_type (void) 
@@ -295,9 +228,6 @@ config_log_destroy (GtkObject *object)
 	config_log = CONFIG_LOG (object);
 
 	do_unload (config_log, !config_log->p->deleted);
-#if 0
-	disconnect_socket (config_log);
-#endif
 
 	GTK_OBJECT_CLASS (parent_class)->destroy (GTK_OBJECT (config_log));
 }
@@ -334,9 +264,6 @@ config_log_open (Location *location)
 
 	config_log_reset_filenames (CONFIG_LOG (object));
 	do_load (CONFIG_LOG (object));
-#if 0
-	connect_socket (CONFIG_LOG (object));
-#endif
 
 	return object;
 }
@@ -355,14 +282,11 @@ config_log_delete (ConfigLog *config_log)
 	g_return_if_fail (config_log != NULL);
 	g_return_if_fail (IS_CONFIG_LOG (config_log));
 
-	if (config_log->p->file_buffer != NULL) {
-		io_buffer_destroy (config_log->p->file_buffer);
-		config_log->p->file_buffer = NULL;
-	}
-
 	if (config_log->p->filename != NULL)
 		unlink (config_log->p->filename);
 
+	do_unload (config_log, FALSE);
+
 	config_log->p->deleted = TRUE;
 	gtk_object_destroy (GTK_OBJECT (config_log));
 }
@@ -375,7 +299,7 @@ config_log_delete (ConfigLog *config_log)
 gint 
 config_log_get_rollback_id_for_date (ConfigLog *config_log,
 				     struct tm *date,
-				     gchar *backend_id) 
+				     const gchar *backend_id) 
 {
 	GList *node;
 
@@ -385,8 +309,7 @@ config_log_get_rollback_id_for_date (ConfigLog *config_log,
 
 	if (config_log->p->log_data == NULL)
 		config_log->p->log_data = 
-			load_log_entry (config_log, FALSE,
-					config_log->p->file_buffer, NULL);
+			load_log_entry (config_log, NULL);
 
 	if (date == NULL)
 		node = config_log->p->log_data;
@@ -409,7 +332,8 @@ config_log_get_rollback_id_for_date (ConfigLog *config_log,
 
 gint
 config_log_get_rollback_id_by_steps (ConfigLog *config_log,
-				     guint steps, gchar *backend_id)
+				     guint steps,
+				     const gchar *backend_id)
 {
 	GList *node;
 
@@ -420,8 +344,7 @@ config_log_get_rollback_id_by_steps (ConfigLog *config_log,
 	node = config_log->p->log_data;
 
 	if (node == NULL)
-		node = load_log_entry (config_log, FALSE,
-				       config_log->p->file_buffer, node);
+		node = load_log_entry (config_log, node);
 
 	while (node != NULL && steps-- > 0) {
 		node = find_config_log_entry_backend
@@ -433,8 +356,7 @@ config_log_get_rollback_id_by_steps (ConfigLog *config_log,
 		if (steps > 0) {
 			if (node->next == NULL)
 				node = load_log_entry
-					(config_log, FALSE,
-					 config_log->p->file_buffer, node);
+					(config_log, node);
 			else
 				node = node->next;
 		}
@@ -459,8 +381,7 @@ config_log_get_backend_id_for_id (ConfigLog *config_log, gint id)
 
 	if (config_log->p->log_data == NULL)
 		config_log->p->log_data = 
-			load_log_entry (config_log, FALSE,
-					config_log->p->file_buffer, NULL);
+			load_log_entry (config_log, NULL);
 
 	node = find_config_log_entry_id (config_log,
 					 config_log->p->log_data, id);
@@ -473,7 +394,7 @@ config_log_get_backend_id_for_id (ConfigLog *config_log, gint id)
 
 /* Return the date the data with the given id was written */
 
-struct tm *
+const struct tm *
 config_log_get_date_for_id (ConfigLog *config_log, gint id) 
 {
 	GList *node;
@@ -484,8 +405,7 @@ config_log_get_date_for_id (ConfigLog *config_log, gint id)
 
 	if (config_log->p->log_data == NULL)
 		config_log->p->log_data = 
-			load_log_entry (config_log, FALSE,
-					config_log->p->file_buffer, NULL);
+			load_log_entry (config_log, NULL);
 
 	node = find_config_log_entry_id (config_log,
 					 config_log->p->log_data, id);
@@ -509,8 +429,9 @@ config_log_get_date_for_id (ConfigLog *config_log, gint id)
  **/
 
 gint 
-config_log_write_entry (ConfigLog *config_log, gchar *backend_id,
-			gboolean is_default_data) 
+config_log_write_entry (ConfigLog   *config_log,
+			const gchar *backend_id,
+			gboolean     is_default_data) 
 {
 	ConfigLogEntry *entry;
 
@@ -533,19 +454,7 @@ config_log_write_entry (ConfigLog *config_log, gchar *backend_id,
 	config_log->p->log_data =
 		g_list_prepend (config_log->p->log_data, entry);
 
-#if 0
-	if (config_log->p->socket_owner) {
-		slave_broadcast_data (NULL, config_log);
-#endif
-		dump_log (config_log);
-		if (config_log->p->file_buffer)
-			io_buffer_destroy (config_log->p->file_buffer);
-		do_load (config_log);
-#if 0
-	} else {
-		write_log (config_log->p->socket_buffer, entry);
-	}
-#endif
+	dump_log (config_log);
 
 	return entry->id;
 }
@@ -578,9 +487,7 @@ config_log_iterate (ConfigLog *config_log, ConfigLogIteratorCB callback,
 			      entry->date, data)) break;
 
 		if (node->next == NULL) 
-			node = load_log_entry (config_log, FALSE,
-					       config_log->p->file_buffer,
-					       node);
+			node = load_log_entry (config_log, node);
 		else
 			node = node->next;
 	}
@@ -606,33 +513,10 @@ config_log_reset_filenames (ConfigLog *config_log)
 		g_concat_dir_and_file (location_get_path
 				       (config_log->p->location),
 				       "config.log");
-
-	if (config_log->p->socket_filename != NULL) {
-		if (config_log->p->socket_owner)
-			unlink (config_log->p->socket_filename);
-
-		g_free (config_log->p->socket_filename);
-	}
-
-	config_log->p->socket_filename = 
-		g_concat_dir_and_file (location_get_path 
-				       (config_log->p->location),
-				       "config.log.socket");
-
-	if (config_log->p->socket_owner) {
-		if (check_socket_filename (config_log))
-			bind_socket (config_log,
-				     g_io_channel_unix_get_fd
-				     (config_log->p->socket_buffer->channel),
-				     FALSE);
-		else
-			g_warning ("Could not rebind socket after "
-				   "reseting filenames");
-	}
 }
 
 /**
- * config_log_reload
+ * config_log_reload:
  * @config_log:
  *
  * Reloads the entire config log, throwing out any newly created entries
@@ -649,6 +533,97 @@ config_log_reload (ConfigLog *config_log)
 	do_load (config_log);
 }
 
+/**
+ * config_log_garbage_collect:
+ * @config_log:
+ * @backend_id: Backend on which to iterate
+ * @callback: Callback to issue on any log entry to be culled
+ * @data: Arbitrary data to pass to callback
+ *
+ * Iterates through the configuration log and culls excess entries for the given
+ * backend.
+ *
+ * The algorithm we use is the following: We scan entries in temporal order. For
+ * each consecutive pair of entries, let t1 be the time at which the former was
+ * made and t2 be the time of the latter. If K_CONST * (t2 - t1) < t2, then we
+ * delete the former entry and issue the callback. We select K_CONST
+ * appropriately, i.e. a user will likely not want to keep entries separated by
+ * under five minutes for very long, while she may want to keep entries
+ * separated by two weeks for much longer.
+ */
+
+#define K_CONST 15
+
+void
+config_log_garbage_collect (ConfigLog        *config_log,
+			    gchar            *backend_id,
+			    GarbageCollectCB  callback,
+			    gpointer          data)
+{
+	GList *node, *list = NULL;
+	ConfigLogEntry *e1, *e2;
+	time_t t1, t2, now;
+	struct tm now_b, *tmp_date;
+
+	g_return_if_fail (config_log != NULL);
+	g_return_if_fail (IS_CONFIG_LOG (config_log));
+
+	if (config_log->p->log_data == NULL)
+		config_log->p->log_data = 
+			load_log_entry (config_log, NULL);
+
+	node = config_log->p->log_data;
+
+	/* We build a list of config log nodes to facilitate removing the nodes
+	 * from the main cache at a later point
+	 */
+
+	while (1) {
+		node = find_config_log_entry_backend
+			(config_log, node, backend_id);
+
+		if (node == NULL)
+			break;
+
+		list = g_list_prepend (list, node);
+		node = node->next;
+	}
+
+	if (list == NULL) return;
+
+	now = time (NULL);
+	gmtime_r (&now, &now_b);
+	now = mktime (&now_b);
+
+	for (node = list; node->next != NULL; node = node->next) {
+		e1 = ((GList *) node->data)->data;
+		e2 = ((GList *) node->next->data)->data;
+
+		tmp_date = dup_date (e1->date);
+		t1 = mktime (tmp_date);
+		g_free (tmp_date);
+
+		tmp_date = dup_date (e2->date);
+		t2 = mktime (tmp_date);
+		g_free (tmp_date);
+
+		if (now < t2 || now < t1)
+			g_warning ("Log entry is in the future!");
+		if (t1 > t2)
+			g_warning ("Log entries are out of order!");
+
+		if (K_CONST * difftime (t2, t1) < difftime (now, t2)) {
+			config_log->p->log_data =
+				g_list_remove_link (config_log->p->log_data, node->data);
+			callback (config_log, backend_id, e1->id, data);
+		}
+	}
+
+	g_list_free (list);
+
+	config_log->p->first_old = NULL;
+}
+
 /* Find the config log entry with the id given, starting at the given
  * node. Return a pointer to the node.
  */
@@ -676,8 +651,7 @@ find_config_log_entry_id (ConfigLog *config_log, GList *start, gint id)
 	}
 
 	while (1) {
-		start = load_log_entry (config_log, FALSE,
-					config_log->p->file_buffer, last);
+		start = load_log_entry (config_log, last);
 		if (start == NULL) return NULL;
 		entry = (ConfigLogEntry *) start->data;
 		if (entry->id == id)
@@ -715,8 +689,7 @@ find_config_log_entry_date (ConfigLog *config_log, GList *start,
 	}
 
 	while (1) {
-		start = load_log_entry (config_log, FALSE,
-					config_log->p->file_buffer, last);
+		start = load_log_entry (config_log, last);
 		if (start == NULL) return NULL;
 		entry = (ConfigLogEntry *) start->data;
 		if (time_geq (date, entry->date))
@@ -731,8 +704,9 @@ find_config_log_entry_date (ConfigLog *config_log, GList *start,
  */
 
 static GList *
-find_config_log_entry_backend (ConfigLog *config_log, GList *start, 
-			       gchar *backend_id) 
+find_config_log_entry_backend (ConfigLog *config_log,
+			       GList *start, 
+			       const gchar *backend_id) 
 {
 	GList *last;
 	ConfigLogEntry *entry;
@@ -752,8 +726,7 @@ find_config_log_entry_backend (ConfigLog *config_log, GList *start,
 	}
 
 	while (1) {
-		start = load_log_entry (config_log, FALSE,
-					config_log->p->file_buffer, last);
+		start = load_log_entry (config_log, last);
 		if (start == NULL) return NULL;
 		entry = (ConfigLogEntry *) start->data;
 		if (!strcmp (entry->backend_id, backend_id))
@@ -763,25 +736,25 @@ find_config_log_entry_backend (ConfigLog *config_log, GList *start,
 	return NULL;
 }
 
-/* Loads a log entry from the given file and attaches it to the beginning or
- * the end of the config log */
+/* Loads a log entry from the given file and attaches it to the end of the config log */
 
 static GList *
-load_log_entry (ConfigLog *config_log, gboolean from_socket,
-		IOBuffer *input, GList *last) 
+load_log_entry (ConfigLog *config_log,
+		GList     *last) 
 {
-	gchar *buffer, *backend_id;
+	gchar *backend_id;
+	gchar buffer[1024];
 	ConfigLogEntry *entry;
 	gboolean success;
 
 	g_return_val_if_fail (config_log != NULL, NULL);
 	g_return_val_if_fail (IS_CONFIG_LOG (config_log), NULL);
 
-	if (input == NULL || input->closed) return NULL;
+	if (config_log->p->file_stream == NULL || feof (config_log->p->file_stream))
+		return NULL;
 
-	buffer = io_buffer_read_line (input);
-
-	if (buffer == NULL) return NULL;
+	if (fgets (buffer, 1024, config_log->p->file_stream) == NULL)
+		return NULL;
 
 	entry = g_new0 (ConfigLogEntry, 1);
 	entry->date = g_new0 (struct tm, 1);
@@ -791,22 +764,15 @@ load_log_entry (ConfigLog *config_log, gboolean from_socket,
 	if (success) {
 		entry->backend_id = g_strdup (backend_id);
 
-		if (from_socket) {
-			config_log->p->log_data = 
-				g_list_prepend (config_log->p->log_data, entry);
+		last = g_list_append (last, entry);
 
-			return config_log->p->log_data;
-		} else {
-			last = g_list_append (last, entry);
+		if (config_log->p->log_data == NULL)
+			config_log->p->log_data = last;
 
-			if (config_log->p->log_data == NULL)
-				config_log->p->log_data = last;
+		if (config_log->p->first_old == NULL)
+			config_log->p->first_old = last;
 
-			if (config_log->p->first_old == NULL)
-				config_log->p->first_old = last;
-
-			return g_list_find (last, entry);
-		}
+		return g_list_find (last, entry);
 	} else {
 		g_free (entry->date);
 		g_free (entry);
@@ -827,6 +793,8 @@ load_log_entry (ConfigLog *config_log, gboolean from_socket,
 static gboolean
 parse_line (char *buffer, int *id, struct tm *date, char **backend_id) 
 {
+	unsigned int len;
+
 	sscanf (buffer, "%x", id);
 
 	while (isxdigit (*buffer)) buffer++;
@@ -856,9 +824,16 @@ parse_line (char *buffer, int *id, struct tm *date, char **backend_id)
 	if (extract_number (&buffer, &date->tm_sec, 2) == FALSE)
 		return FALSE;
 
+	date->tm_gmtoff = 0;
+	date->tm_zone = "GMT";
+
 	if (!isspace (*buffer) || *(buffer + 1) == '\0') return FALSE;
 	buffer++;
 
+	len = strlen (buffer);
+	if (buffer[len - 1] == '\n')
+		buffer[len - 1] = '\0';
+
 	*backend_id = buffer;
 
 	return TRUE;
@@ -895,20 +870,11 @@ time_geq (struct tm *time1, struct tm *time2)
 static gboolean
 do_load (ConfigLog *config_log) 
 {
-	int fd;
-
 	g_return_val_if_fail (config_log != NULL, FALSE);
 	g_return_val_if_fail (IS_CONFIG_LOG (config_log), FALSE);
-	g_return_val_if_fail (config_log->p->location != NULL, FALSE);
-	g_return_val_if_fail (IS_LOCATION (config_log->p->location), FALSE);
 
-	fd = open (config_log->p->filename, O_RDONLY);
-
-	if (fd != -1)
-		config_log->p->file_buffer = 
-			io_buffer_new (g_io_channel_unix_new (fd), FALSE);
-	else
-		config_log->p->file_buffer = NULL;
+	config_log->p->file_stream = fopen (config_log->p->filename, "r");
+	config_log->p->first_old = NULL;
 
 	return TRUE;
 }
@@ -920,16 +886,16 @@ do_load (ConfigLog *config_log)
 static void
 do_unload (ConfigLog *config_log, gboolean write_log) 
 {
-	GList *tmp;
-
 	g_return_if_fail (config_log != NULL);
 	g_return_if_fail (IS_CONFIG_LOG (config_log));
 
+	if (config_log->p->deleted) return;
+
 	if (write_log) dump_log (config_log);
 
-	if (config_log->p->file_buffer) {
-		io_buffer_destroy (config_log->p->file_buffer);
-		config_log->p->file_buffer = NULL;
+	if (config_log->p->file_stream) {
+		fclose (config_log->p->file_stream);
+		config_log->p->file_stream = NULL;
 	}
 
 	if (config_log->p->filename) {
@@ -937,21 +903,9 @@ do_unload (ConfigLog *config_log, gboolean write_log)
 		config_log->p->filename = NULL;
 	}
 
-	if (config_log->p->socket_filename) {
-		if (config_log->p->socket_owner)
-			unlink (config_log->p->socket_filename);
-
-		g_free (config_log->p->socket_filename);
-		config_log->p->socket_filename = NULL;
-	}
-
-	while (config_log->p->log_data != NULL) {
-		tmp = config_log->p->log_data->next;
-		config_log_entry_destroy
-			((ConfigLogEntry *) config_log->p->log_data->data);
-		g_list_free_1 (config_log->p->log_data);
-		config_log->p->log_data = tmp;
-	}
+	g_list_foreach (config_log->p->log_data, (GFunc) config_log_entry_destroy, NULL);
+	g_list_free (config_log->p->log_data);
+	config_log->p->log_data = NULL;
 }
 
 /* Returns the next id number in the sequence */
@@ -960,8 +914,7 @@ static gint
 get_next_id (ConfigLog *config_log) 
 {
 	if (config_log->p->log_data == NULL) {
-		if (load_log_entry (config_log, FALSE,
-				    config_log->p->file_buffer, NULL) == NULL)
+		if (load_log_entry (config_log, NULL) == NULL)
 			return 0;
 	}
 
@@ -982,47 +935,41 @@ static struct tm *
 get_current_date (void) 
 {
 	time_t current_time;
-	struct tm *time_1, *ret;
+	struct tm time_2, *time_1 = &time_2, *ret;
 
 	current_time = time (NULL);
-	time_1 = localtime (&current_time);
+	gmtime_r (&current_time, time_1);
 	ret = g_new (struct tm, 1);
 	memcpy (ret, time_1, sizeof (struct tm));
 	return ret;
 }
 
-/* Write out a log entry */
+/* Write out a single log entry */
 
 static void
-write_log (IOBuffer *output, ConfigLogEntry *entry) 
+write_log (FILE *output, ConfigLogEntry *entry) 
 {
-	gchar *str;
-
 	g_return_if_fail (output != NULL);
 	g_return_if_fail (entry != NULL);
 	g_return_if_fail (entry->id >= 0);
 	g_return_if_fail (entry->date != NULL);
 	g_return_if_fail (entry->backend_id != NULL);
 
-	str = g_strdup_printf ("%08x %04d%02d%02d %02d:%02d:%02d %s\n",
-			       entry->id, entry->date->tm_year + 1900, 
-			       entry->date->tm_mon + 1, entry->date->tm_mday, 
-			       entry->date->tm_hour, entry->date->tm_min, 
-			       entry->date->tm_sec, entry->backend_id);
-	DEBUG_MSG ("Writing %s, from_socket = %d", str, output->from_socket);
-	io_buffer_write (output, str);
-	g_free (str);
+	fprintf (output, "%08x %04d%02d%02d %02d:%02d:%02d %s\n",
+		 entry->id, entry->date->tm_year + 1900, 
+		 entry->date->tm_mon + 1, entry->date->tm_mday, 
+		 entry->date->tm_hour, entry->date->tm_min, 
+		 entry->date->tm_sec, entry->backend_id);
 }
 
+/* Writes out the entire current configuration log to the disk */
+
 static void 
 dump_log (ConfigLog *config_log) 
 {
-	char *filename_out;
+	char  *filename_out;
 	GList *first;
-	int out_fd;
-	IOBuffer *output;
-
-	DEBUG_MSG ("Enter");
+	FILE  *input, *output;
 
 	g_return_if_fail (config_log != NULL);
 	g_return_if_fail (IS_CONFIG_LOG (config_log));
@@ -1030,38 +977,40 @@ dump_log (ConfigLog *config_log)
 	g_return_if_fail (IS_LOCATION (config_log->p->location));
 	g_return_if_fail (location_get_path (config_log->p->location) != NULL);
 
+	if (config_log->p->deleted) return;
+
 	filename_out = g_concat_dir_and_file (location_get_path
 					      (config_log->p->location),
 					      "config.log.out");
 
-	out_fd = open (filename_out, O_CREAT | O_WRONLY | O_TRUNC, 0600);
+	output = fopen (filename_out, "w");
 
-	if (out_fd == -1) {
+	if (output == NULL) {
 		g_critical ("Could not open output file: %s",
 			    g_strerror (errno));
 		return;
 	}
 
-	output = io_buffer_new (g_io_channel_unix_new (out_fd), FALSE);
-
 	for (first = config_log->p->log_data;
 	     first != config_log->p->first_old; 
 	     first = first->next)
 		write_log (output, first->data);
 
-	config_log->p->first_old = config_log->p->log_data;
-
-	if (config_log->p->file_buffer) {
-		io_buffer_rewind (config_log->p->file_buffer);
-		io_buffer_dump (config_log->p->file_buffer, output);
+	if (config_log->p->file_stream != NULL &&
+	    ((config_log->p->first_old == NULL && config_log->p->log_data == NULL) ||
+	     (config_log->p->first_old != NULL && config_log->p->log_data != NULL)))
+	{
+		input = fopen (config_log->p->filename, "r");
+		dump_file (input, output);
+		fclose (input);
 	}
 
-	io_buffer_destroy (output);
+	config_log->p->first_old = config_log->p->log_data;
+
+	fclose (output);
 
 	if (config_log->p->filename)
 		rename (filename_out, config_log->p->filename);
-
-	DEBUG_MSG ("Exit");
 }
 
 /* Return TRUE if the config log has entries made by actual configuration
@@ -1075,8 +1024,7 @@ has_nondefaults (ConfigLog *config_log)
 	ConfigLogEntry *first;
 
 	if (config_log->p->log_data == NULL)
-		load_log_entry (config_log, FALSE, config_log->p->file_buffer,
-				config_log->p->log_data);
+		load_log_entry (config_log, config_log->p->log_data);
 
 	if (config_log->p->log_data == NULL)
 		return FALSE;
@@ -1089,227 +1037,7 @@ has_nondefaults (ConfigLog *config_log)
 		return TRUE;
 }
 
-/* Try to connect to the synchronization socket for this configuration log. If
- * no socket exists, take ownership of the config log and listen for slave
- * connections.
- */
-
-static gboolean
-connect_socket (ConfigLog *config_log)
-{
-	int fd, flags;
-
-	config_log->p->socket_buffer = NULL;
-	config_log->p->socket_owner = check_socket_filename (config_log);
-
-	fd = socket (PF_UNIX, SOCK_STREAM, 0);
-
-	if (fd < 0) {
-		g_warning ("Could not create socket: %s", g_strerror (errno));
-		return FALSE;
-	}
-
-	if (!bind_socket (config_log, fd, TRUE)) {
-		close (fd);
-		return FALSE;
-	}
-
-	config_log->p->socket_buffer =
-		io_buffer_new (g_io_channel_unix_new (fd), TRUE);
-
-	if (config_log->p->socket_owner) {
-		config_log->p->input_id = 
-			g_io_add_watch (config_log->p->socket_buffer->channel,
-					G_IO_IN | G_IO_ERR,
-					(GIOFunc) socket_connect_cb,
-					config_log);
-	} else {
-		DEBUG_MSG ("Adding watch to listen for data"); 
-
-		config_log->p->input_id = 
-			g_io_add_watch (config_log->p->socket_buffer->channel,
-					G_IO_IN | G_IO_ERR | G_IO_HUP,
-					(GIOFunc) socket_data_cb,
-					config_log);
-
-		/* Read any data that might have come through before we added
-		 * the watch */
-		flags = fcntl (fd, F_GETFL);
-
-		if (flags != -1) {
-			flags |= O_NONBLOCK;
-			fcntl (fd, F_SETFL, flags);
-		}
-
-		socket_data_cb (config_log->p->socket_buffer->channel,
-				G_IO_IN, config_log);
-	}
-
-	return config_log->p->socket_owner;
-}
-
-/* Checks to see if the filename associated with the socket is free. Returns
- * TRUE if it is and FALSE otherwise */
-
-static gboolean
-check_socket_filename (ConfigLog *config_log) 
-{
-	struct stat buf;
-	gboolean is_free;
-
-	if (config_log->p->socket_filename == NULL) return FALSE;
-
-	if (stat (config_log->p->socket_filename, &buf) == -1) {
-		if (errno == ENOENT) {
-			is_free = TRUE;
-		} else {
-			g_warning ("Could not stat file: %s",
-				   g_strerror (errno));
-			return FALSE;
-		}
-	} else {
-		is_free = FALSE;
-	}
-
-	if (!is_free && !S_ISSOCK (buf.st_mode)) {
-		g_warning ("There is another file in the way of the socket");
-		return FALSE;
-	}
-
-	return is_free;
-}
-
-/* Binds (or rebinds) the given socket to the correct filename; optionally
- * connects to the correct socket if the current object is not the config log
- * owner
- *
- * If it cannot connect to the given socket, this means the process owner has
- * died unexpectedly and left it lying around. It then tries to take ownership
- * of it.
- */
-
-static gboolean
-bind_socket (ConfigLog *config_log, int fd, gboolean do_connect) 
-{
-	struct sockaddr_un name;
-
-	/* FIXME: What if the socket filename is too long here? */
-	if (config_log->p->socket_owner || do_connect) {
-		name.sun_family = AF_UNIX;
-		strncpy (name.sun_path, config_log->p->socket_filename,
-			 sizeof (name.sun_path));
-	}
-
-	if (do_connect && !config_log->p->socket_owner) {
-		DEBUG_MSG ("Trying to connect to socket");
-
-		if (!connect (fd, (struct sockaddr *) &name, SUN_LEN (&name)))
-			return TRUE;
-
-		if (errno != ECONNREFUSED) {
-			g_warning ("Could not connect to socket: %s",
-				   g_strerror (errno));
-			return FALSE;
-		} else {
-			unlink (config_log->p->socket_filename);
-			config_log->p->socket_owner = TRUE;
-		}
-	}
-
-	if (bind (fd, (struct sockaddr *) &name, SUN_LEN (&name)) < 0) {
-		g_warning ("Could not bind to socket filename: %s",
-			   g_strerror (errno));
-		return FALSE;
-	}
-
-	if (listen (fd, 5) < 0) {
-		g_warning ("Could not set up socket for listening: %s",
-			   g_strerror (errno));
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-static void
-disconnect_socket (ConfigLog *config_log) 
-{
-	if (config_log->p->socket_owner)
-		g_list_foreach (config_log->p->slaves,
-				(GFunc) slave_destroy, NULL);
-
-	g_source_remove (config_log->p->input_id);
-
-	if (config_log->p->socket_buffer != NULL)
-		io_buffer_destroy (config_log->p->socket_buffer);
-}
-
-static gboolean
-socket_connect_cb (GIOChannel *channel, GIOCondition condition,
-		   ConfigLog *config_log) 
-{
-	int fd;
-	struct sockaddr_un addr;
-	socklen_t len;
-
-	DEBUG_MSG ("Enter");
-
-	g_return_val_if_fail (config_log != NULL, FALSE);
-	g_return_val_if_fail (IS_CONFIG_LOG (config_log), FALSE);
-
-	if (condition == G_IO_IN) {
-		fd = accept (g_io_channel_unix_get_fd
-			     (config_log->p->socket_buffer->channel),
-			     &addr, &len);
-
-		if (fd < 0) {
-			g_warning ("Could not accept connection: %s",
-				   g_strerror (fd));
-			return TRUE;
-		}
-
-		config_log->p->slaves =
-			g_list_prepend (config_log->p->slaves,
-					slave_new (config_log, fd));
-	}
-
-	DEBUG_MSG ("Exit");
-
-	return TRUE;
-}
-
-/* Callback issued when data comes in over the socket; only used when this is
- * *not* the master socket. */
-
-static gboolean
-socket_data_cb (GIOChannel *channel, GIOCondition condition,
-		ConfigLog *config_log)
-{
-	g_return_val_if_fail (config_log != NULL, FALSE);
-	g_return_val_if_fail (IS_CONFIG_LOG (config_log), FALSE);
-
-	DEBUG_MSG ("Enter");
-
-	if (condition & G_IO_HUP) {
-		DEBUG_MSG ("Connection closing");
-		disconnect_socket (config_log);
-		connect_socket (config_log);
-		return FALSE;
-	}
-	else if (condition & G_IO_IN) {
-		if (load_log_entry (config_log, TRUE,
-				    config_log->p->socket_buffer, NULL) == NULL) {
-			DEBUG_MSG ("Connection closing");
-			disconnect_socket (config_log);
-			connect_socket (config_log);
-			return FALSE;
-		}
-	}
-
-	DEBUG_MSG ("Exit");
-
-	return TRUE;
-}
+/* Deallocates the given config log entry structure */
 
 static void 
 config_log_entry_destroy (ConfigLogEntry *entry) 
@@ -1323,228 +1051,19 @@ config_log_entry_destroy (ConfigLogEntry *entry)
 	g_free (entry);
 }
 
-static Slave *
-slave_new (ConfigLog *config_log, int fd) 
-{
-	Slave *slave;
-
-	slave = g_new0 (Slave, 1);
-	slave->config_log = config_log;
-	slave->buffer = io_buffer_new (g_io_channel_unix_new (fd), TRUE);
-	slave->source_id = g_io_add_watch (slave->buffer->channel,
-					   G_IO_IN | G_IO_ERR | G_IO_HUP,
-					   (GIOFunc) slave_data_cb,
-					   slave);
-
-	return slave;
-}
+/* Dumps the entire contents from one stream into another */
 
 static void
-slave_destroy (Slave *slave) 
+dump_file (FILE *input, FILE *output) 
 {
-	g_return_if_fail (slave != NULL);
-	g_return_if_fail (slave->config_log != NULL);
-	g_return_if_fail (IS_CONFIG_LOG (slave->config_log));
+	char buffer[4096];
+	size_t len;
 
-	slave->config_log->p->slaves =
-		g_list_remove (slave->config_log->p->slaves, slave);
-	io_buffer_destroy (slave->buffer);
-	g_free (slave);
-}
+	g_return_if_fail (input != NULL);
+	g_return_if_fail (output != NULL);
 
-static gboolean
-slave_data_cb (GIOChannel *channel, GIOCondition condition,
-	       Slave *slave)
-{
-	DEBUG_MSG ("Enter");
-
-	g_return_val_if_fail (slave != NULL, FALSE);
-	g_return_val_if_fail (slave->config_log != NULL, FALSE);
-	g_return_val_if_fail (IS_CONFIG_LOG (slave->config_log), FALSE);
-
-	DEBUG_MSG ("Condition is %d", condition);
-
-	if (condition & G_IO_HUP || slave->buffer->closed) {
-		DEBUG_MSG ("Removing slave");
-		slave_destroy (slave);
-		return FALSE;
-	}
-	else if (condition & G_IO_IN) {
-		if (load_log_entry (slave->config_log, TRUE, slave->buffer,
-				    NULL) != NULL) 
-		{
-			slave_broadcast_data (slave, slave->config_log);
-			dump_log (slave->config_log);
-			if (slave->config_log->p->file_buffer)
-				io_buffer_destroy
-					(slave->config_log->p->file_buffer);
-			do_load (slave->config_log);
-		}
-	}
-
-	DEBUG_MSG ("Exit");
-
-	return TRUE;
-}
-
-/* Broadcasts first log entry to all the slaves of the config log besides the
- * one given */
-
-static void
-slave_broadcast_data (Slave *slave, ConfigLog *config_log) 
-{
-	GList *node;
-	Slave *current;
-	ConfigLogEntry *first_entry;
-
-	DEBUG_MSG ("Enter");
-
-	first_entry = config_log->p->log_data->data;
-
-	for (node = config_log->p->slaves; node != NULL; node = node->next) {
-		current = node->data;
-
-		if (current == slave) continue;
-		write_log (current->buffer, first_entry);
-	}
-
-	DEBUG_MSG ("Exit");
-}
-
-static IOBuffer *
-io_buffer_new (GIOChannel *channel, gboolean from_socket)
-{
-	IOBuffer *buffer;
-
-	buffer = g_new0 (IOBuffer, 1);
-	buffer->channel = channel;
-	buffer->read_ptr = buffer->write_ptr = buffer->buffer;
-	buffer->from_socket = from_socket;
-
-	return buffer;
-}
-
-static void
-io_buffer_destroy (IOBuffer *buffer)
-{
-	g_io_channel_close (buffer->channel);
-	g_io_channel_unref (buffer->channel);
-	g_free (buffer);
-}
-
-/* Note: The two functions below are borrowed from GDict */
-
-/* io_buffer_cycle ()
- *
- * Reads additional data from the socket if it can
- *
- * Returns 0 on success, 1 if there is no more data to read, -1 on socket error
- */
-
-static int 
-io_buffer_cycle (IOBuffer *buffer) 
-{
-	int amt_read, res;
-    
-	if (!buffer->read_cycle_done && !buffer->closed) {
-		if (buffer->write_ptr > buffer->read_ptr)
-			memmove (buffer->buffer, buffer->read_ptr,
-				 buffer->write_ptr - buffer->read_ptr);
-		buffer->write_ptr -= buffer->read_ptr - buffer->buffer;
-		buffer->read_ptr = buffer->buffer;
-
-		res = g_io_channel_read (buffer->channel, buffer->write_ptr,
-					 buffer->buffer + READ_BUF_LEN 
-					 - buffer->write_ptr, 
-					 &amt_read);
-
-		if (res == G_IO_ERROR_AGAIN)
-			buffer->read_cycle_done = TRUE;
-		else if (amt_read == 0)
-			buffer->closed = TRUE;
-		else if (res != G_IO_ERROR_NONE)
-			return -1;
-
-		buffer->write_ptr += amt_read;
-		return 0;
-	}
-	else {
-		return 1;
+	while (!feof (input)) {
+		len = fread (buffer, sizeof (char), 4096, input);
+		fwrite (buffer, sizeof (char), len, output);
 	}
 }
-
-/* io_buffer_read_line ()
- *
- * Reads a line of text from the socket and performs a CR-LF translation
- *
- * Returns a pointer to the string on success
- */
-
-static gchar *
-io_buffer_read_line (IOBuffer *buffer) {
-	gchar *start_ptr, *end_ptr;
-
-	if (!buffer->channel) return NULL;
-
-	while (1) {
-		end_ptr = strchr (buffer->read_ptr, '\n');
-
-		if (end_ptr == NULL || end_ptr > buffer->write_ptr) {
-			if (io_buffer_cycle (buffer)) return NULL;
-		}
-		else {
-			break;
-		}
-	}
-
-	end_ptr[0] = '\0';
-
-	start_ptr = buffer->read_ptr;
-	buffer->read_ptr = end_ptr + 1;
-
-	DEBUG_MSG ("Line read was %s; from_socket = %d",
-		   start_ptr, buffer->from_socket);
-
-	return start_ptr;
-}
-
-static void
-io_buffer_write (IOBuffer *buffer, gchar *str) 
-{
-	gint bytes_written;
-
-	g_return_if_fail (buffer != NULL);
-
-	if (str == NULL) return;
-
-	g_io_channel_write (buffer->channel, str, strlen (str),
-			    &bytes_written);
-}
-
-static void
-io_buffer_rewind (IOBuffer *buffer)
-{
-	g_return_if_fail (buffer != NULL);
-
-	g_io_channel_seek (buffer->channel, 0, G_SEEK_SET);
-}
-
-/* Dumps the contents of the source into the destination.
- *
- * Note: The source will be completely screwed up and unusable after this
- * point!!!!
- */
-
-static void
-io_buffer_dump (IOBuffer *source, IOBuffer *dest)
-{
-	gchar buffer[4096];
-	gint written, read;
-
-	g_return_if_fail (buffer != NULL);
-
-	do {
-		g_io_channel_read (source->channel, buffer, 4096, &read);
-		g_io_channel_write (dest->channel, buffer, read, &written);
-	} while (read == 4096);
-}
